@@ -5,8 +5,6 @@
 *
 * PHP version 4, 5
 *
-* LICENSE:  LGPL.
-*
 * @category  Net
 * @package   Net_LDAP
 * @author    Tarjej Huse <tarjei@bergfald.no>
@@ -15,7 +13,7 @@
 * @author    Benedikt Hallinger <beni@php.net>
 * @copyright 2003-2007 Tarjej Huse, Jan Wagner, Del Elson, Benedikt Hallinger
 * @license   http://www.gnu.org/copyleft/lesser.html LGPL
-* @version   CVS: $Id: LDAP.php,v 1.70 2007/10/24 06:29:32 beni Exp $
+* @version   CVS: $Id: LDAP.php,v 1.89 2008/03/19 08:10:54 beni Exp $
 * @link      http://pear.php.net/package/Net_LDAP/
 */
 
@@ -29,6 +27,7 @@ require_once 'LDAP/Entry.php';
 require_once 'LDAP/Search.php';
 require_once 'LDAP/Util.php';
 require_once 'LDAP/Filter.php';
+require_once 'LDAP/LDIF.php';
 
 /**
 *  Error constants for errors that are not LDAP errors.
@@ -38,7 +37,7 @@ define('NET_LDAP_ERROR', 1000);
 /**
 * Net_LDAP Version
 */
-define('NET_LDAP_VERSION', '1.0.0');
+define('NET_LDAP_VERSION', '1.1.1');
 
 /**
 * Net_LDAP - manipulate LDAP servers the right way!
@@ -51,7 +50,7 @@ define('NET_LDAP_VERSION', '1.0.0');
 * @author    Benedikt Hallinger <beni@php.net>
 * @copyright 2003-2007 Tarjej Huse, Jan Wagner, Del Elson, Benedikt Hallinger
 * @license   http://www.gnu.org/copyleft/lesser.html LGPL
-* @version   CVS: $Id: LDAP.php,v 1.70 2007/10/24 06:29:32 beni Exp $
+* @version   CVS: $Id: LDAP.php,v 1.89 2008/03/19 08:10:54 beni Exp $
 * @link      http://pear.php.net/package/Net_LDAP/
 */
 class Net_LDAP extends PEAR
@@ -159,6 +158,9 @@ class Net_LDAP extends PEAR
 
         @$obj = & new Net_LDAP($config);
 
+        // todo? better errorhandling for setConfig()?
+
+        // connect and bind with credentials in config
         $err = $obj->bind();
         if (Net_LDAP::isError($err)) {
             return $err;
@@ -249,12 +251,16 @@ class Net_LDAP extends PEAR
     }
 
     /**
-    * Bind to the ldap-server
+    * Bind or rebind to the ldap-server
     *
     * This function binds with the given dn and password to the server. In case
     * no connection has been made yet, it will be startet and startTLS issued
     * if appropiate.
     *
+    * The internal bind configuration is not being updated, so if you call
+    * bind() without parameters, you can rebind with the credentials
+    * provided at first connecting to the server.
+    *
     * @param string $dn       Distinguished name for binding
     * @param string $password Password for binding
     *
@@ -263,25 +269,51 @@ class Net_LDAP extends PEAR
     */
     function bind($dn = null, $password = null)
     {
-        $msg = $this->_connect();
-        if (Net_LDAP::isError($msg)) {
-            return $msg;
-        }
+        // fetch current bind credentials
         if (is_null($dn)) {
             $dn = $this->_config["binddn"];
         }
         if (is_null($password)) {
             $password = $this->_config["bindpw"];
         }
-        if (is_null($dn)) {
-            $msg = @ldap_bind($this->_link);
+
+        // Connect first, if we haven't so far.
+        // This will also bind us to the server.
+        if ($this->_link === false) {
+            // store old credentials so we can revert them later
+            // then overwrite config with new bind credentials
+            $olddn = $this->_config["binddn"];
+            $oldpw = $this->_config["bindpw"];
+
+            // overwrite bind credentials in config
+            // so _connect() knows about them
+            $this->_config["binddn"] = $dn;
+            $this->_config["bindpw"] = $password;
+
+            // try to connect with provided credentials
+            $msg = $this->_connect();
+
+            // reset to previous config
+            $this->_config["binddn"] = $olddn;
+            $this->_config["bindpw"] = $oldpw;
+
+            // see if bind worked
+            if (Net_LDAP::isError($msg)) {
+                return $msg;
+            }
         } else {
-            $msg = @ldap_bind($this->_link, $dn, $password);
-        }
-        if (false === $msg) {
-            return PEAR::raiseError("Bind failed: " .
-                                    @ldap_error($this->_link),
-                                    @ldap_errno($this->_link));
+            // do the requested bind as we are
+            // asked to bind manually
+            if (is_null($dn)) {
+                $msg = @ldap_bind($this->_link); // anonymous bind
+            } else {
+                $msg = @ldap_bind($this->_link, $dn, $password); // privilegued bind
+            }
+            if (false === $msg) {
+                return PEAR::raiseError("Bind failed: " .
+                                        @ldap_error($this->_link),
+                                        @ldap_errno($this->_link));
+            }
         }
         return true;
     }
@@ -373,37 +405,17 @@ class Net_LDAP extends PEAR
             }
 
             //
-            // Attempt an anonymous bind to see if we can bind to the server
-            // If it fails, we try a privilegued bind since some servers don't
-            // allow anonymous connections at all
+            // Attempt to bind to the server. If we have credentials configured,
+            // we try to use them, otherwiese its an anonymous bind.
             //
-            if (! @ldap_bind($this->_link)) {
-                // Anonymous bind failed, try privilegued bind if we have credentials
-                if ($this->_config['binddn']) {
-                    $msg = $this->bind($this->_config['binddn'], $this->_config['bindpw']);
-                    if (Net_LDAP::isError($msg)) {
-                        // The privilegued bind failed too, discard link and save error msg
-                        $this->_link   = false;
-                        $current_error = $msg;
-                    }
-                } else {
-                    // Failed, so discard the link
-                    // DO NOT attempt to call ldap_close on this connection ID,
-                    // even attempting to do so can be fatal.  Just discard the
-                    // link ID!
-                    $this->_link   = false;
-                    $current_error = PEAR::raiseError("Bind failed: " .
-                                        @ldap_error($this->_link),
-                                        @ldap_errno($this->_link));
-                }
-
-                // Record the host as down and try next host
-                // if the privelegued bind failed or we don't
-                // had credentials and the anonymous bind failed
-                if ($this->_link == false) {
-                    $this->_down_host_list[] = $host;
-                    continue;
-                }
+            $msg = $this->bind();
+            if (Net_LDAP::isError($msg)) {
+                // The bind failed, discard link and save error msg.
+                // Then record the host as down and try next one
+                $this->_link             = false;
+                $current_error           = $msg;
+                $this->_down_host_list[] = $host;
+                continue;
             }
 
             //
@@ -514,9 +526,11 @@ class Net_LDAP extends PEAR
         if (@ldap_add($this->_link, $entry->dn(), $entry->getValues())) {
             // entry successfully added, we should update its $ldap reference
             // in case it is not set so far (fresh entry)
-            if (!$entry->_ldap) {
-                $entry->_ldap =& $this;
+            if (!is_a($entry->getLDAP(), 'Net_LDAP')) {
+                $entry->setLDAP($this);
             }
+            // store, that the entry is present inside the directory
+            $entry->_markAsNew(false);
             return true;
         } else {
              return PEAR::raiseError("Could not add entry " . $entry->dn() . " " .
@@ -548,12 +562,12 @@ class Net_LDAP extends PEAR
         }
         // Recursive delete searches for children and calls delete for them
         if ($recursive) {
-            $result = @ldap_list($this->_link, $dn, '(objectClass=*)', array(null));
+            $result = @ldap_list($this->_link, $dn, '(objectClass=*)', array(null), 0, 0);
             if (@ldap_count_entries($this->_link, $result)) {
                 $subentry = @ldap_first_entry($this->_link, $result);
-                $this->delete(@ldap_get_dn($this->_link, $subentry), $recursive);
+                $this->delete(@ldap_get_dn($this->_link, $subentry), true);
                 while ($subentry = @ldap_next_entry($this->_link, $subentry)) {
-                    $this->delete(@ldap_get_dn($this->_link, $subentry), $recursive);
+                    $this->delete(@ldap_get_dn($this->_link, $subentry), true);
                 }
             }
         }
@@ -634,7 +648,7 @@ class Net_LDAP extends PEAR
 
         if (isset($parms['changes']) && is_array($parms['changes'])) {
             foreach ($parms['changes'] as $action => $value) {
-                $msg = $this->modify($entry->dn(), array($action => $value));
+                $msg = $this->modify($entry, array($action => $value));
                 if (Net_LDAP::isError($msg)) {
                     return $msg;
                 }
@@ -671,6 +685,9 @@ class Net_LDAP extends PEAR
     *        find
     *        always
     *
+    * Please note, that you cannot override server side limitations to sizelimit
+    * and timelimit: You can always only lower a given limit.
+    *
     * @param string                 $base   LDAP searchbase
     * @param string|Net_LDAP_Filter $filter LDAP search filter or a Net_LDAP_Filter object
     * @param array                  $params Array of options
@@ -699,11 +716,13 @@ class Net_LDAP extends PEAR
         (isset($params['attrsonly']))  ? $attrsonly  = $params['attrsonly']  : $attrsonly = 0;
         (isset($params['attributes'])) ? $attributes = $params['attributes'] : $attributes = array();
 
+        // Ensure $attributes to be an array in case only one
+        // attribute name was given as string
         if (!is_array($attributes)) {
-            PEAR::raiseError("The param attributes must be an array!");
+            $attributes = array($attributes);
         }
 
-        // reorganize the attributes array index keys
+        // reorganize the $attributes array index keys
         // sometimes there are problems with not consecutive indexes
         $attributes = array_values($attributes);
 
@@ -735,7 +754,7 @@ class Net_LDAP extends PEAR
                 // Errorcode 32 = no such object, i.e. a nullresult.
                 return $obj = & new Net_LDAP_Search ($search, $this, $attributes);
             } elseif ($err == 4) {
-                // Errorcode 4 = sizelimit exeeded. TODO
+                // Errorcode 4 = sizelimit exeeded.
                 return $obj = & new Net_LDAP_Search ($search, $this, $attributes);
             } elseif ($err == 87) {
                 // bad search filter
@@ -853,33 +872,36 @@ class Net_LDAP extends PEAR
     *
     * @param string $dn The DN of the object to test
     *
-    * @return boolean
+    * @return boolean|Net_LDAP_Error
     */
     function dnExists($dn)
     {
+        if (!is_string($dn)) {
+            return PEAR::raiseError('$dn is expected to be a string but is '.gettype($dn).' '.get_class($dn));
+        }
+
         // make dn relative to parent
         $base = Net_LDAP_Util::ldap_explode_dn($dn, array('casefold' => 'none', 'reverse' => false, 'onlyvalues' => false));
         if (Net_LDAP::isError($base)) {
             return $base;
         }
-        $filter_dn = array_shift($base);
-        // maybe the dn consist of a multivalued RDN, we must build the dn in this case
-        // because the $child-RDN is an array!
-        if (is_array($filter_dn)) {
-            $filter_dn = Net_LDAP_Util::canonical_dn($filter_dn);
+        $entry_rdn = array_shift($base);
+        if (is_array($entry_rdn)) {
+            // maybe the dn consist of a multivalued RDN, we must build the dn in this case
+            // because the $entry_rdn is an array!
+            $filter_dn = Net_LDAP_Util::canonical_dn($entry_rdn);
         }
         $base = Net_LDAP_Util::canonical_dn($base);
 
-        $result = @ldap_list($this->_link, $base, $filter_dn, array(), 1, 1);
+        $result = @ldap_list($this->_link, $base, $entry_rdn, array(), 1, 1);
+        if (@ldap_count_entries($this->_link, $result)) {
+            return true;
+        }
         if (ldap_errno($this->_link) == 32) {
             return false;
         }
         if (ldap_errno($this->_link) != 0) {
-            PEAR::raiseError(ldap_error($this->_link), ldap_errno($this->_link));
-        }
-        if (@ldap_count_entries($this->_link, $result)) {
-            $return = true;
-            return $return;
+            return PEAR::raiseError(ldap_error($this->_link), ldap_errno($this->_link));
         }
         return false;
     }
@@ -904,7 +926,7 @@ class Net_LDAP extends PEAR
         if (Net_LDAP::isError($result)) {
             return $result;
         } elseif ($result->count() == 0) {
-            return PEAR::raiseError('Could not fetch entry: no entry found');
+            return PEAR::raiseError('Could not fetch entry '.$dn.': no entry found');
         }
         $entry = $result->shiftEntry();
         if (false == $entry) {
@@ -921,6 +943,11 @@ class Net_LDAP extends PEAR
     * You can pass an optional Net_LDAP object. In this case, a cross directory
     * move will be performed which deletes the entry in the source (THIS) directory
     * and adds it in the directory $target_ldap.
+    * A cross directory move will switch the Entrys internal LDAP reference so
+    * updates to the entry will go to the new directory.
+    *
+    * Note that if you want to do a cross directory move, you need to
+    * pass an Net_LDAP_Entry object, otherwise the attributes will be empty.
     *
     * @param string|Net_LDAP_Entry &$entry      Entry DN or Entry object
     * @param string                $newdn       New location
@@ -930,10 +957,12 @@ class Net_LDAP extends PEAR
     */
     function move(&$entry, $newdn, $target_ldap = null)
     {
-        if (!is_string($entry)) {
-            $entry = new Net_LDAP_Entry($this, $entry);
+        if (is_string($entry)) {
+            $entry_o = $this->getEntry($entry);
+        } else {
+            $entry_o =& $entry;
         }
-        if (!is_a($entry, 'Net_LDAP_Entry')) {
+        if (!is_a($entry_o, 'Net_LDAP_Entry')) {
             return PEAR::raiseError('Parameter $entry is expected to be a Net_LDAP_Entry object! (If DN was passed, conversion failed)');
         }
         if (null !== $target_ldap && !is_a($target_ldap, 'Net_LDAP')) {
@@ -942,27 +971,32 @@ class Net_LDAP extends PEAR
 
         if ($target_ldap && $target_ldap !== $this) {
             // cross directory move
-            if ($target_ldap->dnExists($entry->dn())) {
+            if (is_string($entry)) {
+                return PEAR::raiseError('Unable to perform cross directory move: operation requires a Net_LDAP_Entry object');
+            }
+            if ($target_ldap->dnExists($newdn)) {
                 return PEAR::raiseError('Unable to perform cross directory move: entry does exist in target directory');
             }
-            $res = $target_ldap->add($entry);
+            $entry_o->dn($newdn);
+            $res = $target_ldap->add($entry_o);
             if (Net_LDAP::isError($res)) {
                 return PEAR::raiseError('Unable to perform cross directory move: '.$res->getMessage().' in target directory');
             }
-            $res = $this->delete();
+            $res = $this->delete($entry_o->currentDN());
             if (Net_LDAP::isError($res)) {
-                $res2 = $target_ldap->delete($entry, true); // undo add
+                $res2 = $target_ldap->delete($entry_o); // undo add
                 if (Net_LDAP::isError($res2)) {
-                    $add_error_string = 'Additionally, the deletion of $entry in target directory failed.';
+                    $add_error_string = 'Additionally, the deletion (undo add) of $entry in target directory failed.';
                 }
-                return PEAR::raiseError('Unable to perform cross directory move: '.$res->getMessage().' in target directory. '.$add_error_string);
+                return PEAR::raiseError('Unable to perform cross directory move: '.$res->getMessage().' in source directory. '.$add_error_string);
             }
+            $entry_o->setLDAP($target_ldap);
             return true;
         } else {
             // local move
-            $entry->dn($newdn);
-            $entry->setLDAP($this);
-            return $entry->update();
+            $entry_o->dn($newdn);
+            $entry_o->setLDAP($this);
+            return $entry_o->update();
         }
     }
 
@@ -970,22 +1004,18 @@ class Net_LDAP extends PEAR
     * Copy an entry to a new location
     *
     * The entry will be immediately copied.
-    * If you pass an Net_LDAP_Entry object, the source entry is not required
-    * to be existent on this LDAP server, which can be used to
-    * copy between directory servers.
+    * Please note that only attributes you have
+    * selected will be copied.
     *
-    * @param string|Net_LDAP_Entry &$entry Entry DN or Entry object
-    * @param string                $newdn  New location
+    * @param Net_LDAP_Entry &$entry Entry object
+    * @param string         $newdn  New FQF-DN of the entry
     *
     * @return Net_LDAP_Error|Net_LDAP_Entry Error Message or reference to the copied entry
     */
     function &copy(&$entry, $newdn)
     {
-        if (!is_string($entry)) {
-            $entry = new Net_LDAP_Entry($this, $entry);
-        }
         if (!is_a($entry, 'Net_LDAP_Entry')) {
-            return PEAR::raiseError('Parameter $entry is expected to be a Net_LDAP_Entry object! (If DN was passed, conversion failed)');
+            return PEAR::raiseError('Parameter $entry is expected to be a Net_LDAP_Entry object!');
         }
 
         $newentry = Net_LDAP_Entry::createFresh($newdn, $entry->getValues());
@@ -1135,6 +1165,7 @@ class Net_LDAP extends PEAR
     *
     * @access public
     * @see rootDse()
+    * @return Net_LDAP_Error|Net_LDAP_RootDSE
     */
     function &root_dse()
     {
@@ -1229,16 +1260,20 @@ class Net_LDAP extends PEAR
     }
 
     /**
-    * Encodes given attributes to UTF8 if needed
+    * Encodes given attributes to UTF8 if needed by schema
     *
     * This function takes attributes in an array and then checks against the schema if they need
     * UTF8 encoding. If that is so, they will be encoded. An encoded array will be returned and
     * can be used for adding or modifying.
     *
+    * $attributes is expected to be an array with keys describing
+    * the attribute names and the values as the value of this attribute:
+    * <code>$attributes = array('cn' => 'foo', 'attr2' => array('mv1', 'mv2'));</code>
+    *
     * @param array $attributes Array of attributes
     *
     * @access public
-    * @return array Array of UTF8 encoded attributes
+    * @return array|Net_LDAP_Error Array of UTF8 encoded attributes or Error
     */
     function utf8Encode($attributes)
     {
@@ -1246,12 +1281,17 @@ class Net_LDAP extends PEAR
     }
 
     /**
-    * Decodes the given attribute values
+    * Decodes the given attribute values if needed by schema
+    *
+    * $attributes is expected to be an array with keys describing
+    * the attribute names and the values as the value of this attribute:
+    * <code>$attributes = array('cn' => 'foo', 'attr2' => array('mv1', 'mv2'));</code>
     *
     * @param array $attributes Array of attributes
     *
     * @access public
-    * @return array Array with decoded attribute values
+    * @see utf8Encode()
+    * @return array|Net_LDAP_Error Array with decoded attribute values or Error
     */
     function utf8Decode($attributes)
     {
@@ -1265,10 +1305,14 @@ class Net_LDAP extends PEAR
     * @param array $function   Function to apply to attribute values
     *
     * @access private
-    * @return array Array of attributes with function applied to values
+    * @return array|Net_LDAP_Error Array of attributes with function applied to values or Error
     */
     function _utf8($attributes, $function)
     {
+        if (!is_array($attributes) || array_key_exists(0, $attributes)) {
+            return PEAR::raiseError('Parameter $attributes is expected to be an associative array');
+        }
+
         if (!$this->_schema) {
             $this->_schema = $this->schema();
         }
