@@ -15,7 +15,7 @@ require_once 'Util.php';
 * @author   Jan Wagner <wagner@netsols.de>
 * @author   Tarjej Huse <tarjei@bergfald.no>
 * @license  http://www.gnu.org/copyleft/lesser.html LGPL
-* @version  CVS: $Id: Entry.php,v 1.47 2007/10/24 06:29:32 beni Exp $
+* @version  CVS: $Id: Entry.php,v 1.54 2008/02/27 08:59:42 beni Exp $
 * @link     http://pear.php.net/package/Net_LDAP/
 
 */
@@ -157,9 +157,13 @@ class Net_LDAP_Entry extends PEAR
     /**
     * Creates a fresh entry that may be added to the directory later on
     *
-    * The method should be called statically: $entry = Net_LDAP_Entry::createFresh();
-    *
     * Use this method, if you want to initialize a fresh entry.
+    *
+    * The method should be called statically: $entry = Net_LDAP_Entry::createFresh();
+    * You should put a 'objectClass' attribute into the $attrs so the directory server
+    * knows which object you want to create. However, you may omit this in case you
+    * don't want to add this entry to a directory server.
+    *
     * The attributes parameter is as following:
     * <code>
     * $attrs = array( 'attribute1' => array('value1', 'value2'),
@@ -190,8 +194,11 @@ class Net_LDAP_Entry extends PEAR
     * If you provide an DN, this entry is moved to the new location specified if a DN existed.
     * If the DN was not set, the DN gets initialized. Call {@link update()} to actually create
     * the new Entry in the directory.
+    * To fetch the current active DN after setting a new DN but before an update(), you can use
+    * {@link currentDN()} to retrieve the DN that is currently active.
     *
     * Please note that special characters (eg german umlauts) should be encoded using utf8_encode().
+    * You may use {@link Net_LDAP_Util::canonical_dn()} for properly encoding of the DN.
     *
     * @param string $dn New distinguished name
     *
@@ -206,10 +213,23 @@ class Net_LDAP_Entry extends PEAR
             } else {
                 $this->_newdn = $dn;
             }
-            $return = true;
-            return $return;
+            return true;
         }
-        return (isset($this->_newdn) ? $this->_newdn : $this->_dn);
+        return (isset($this->_newdn) ? $this->_newdn : $this->currentDN());
+    }
+
+    /**
+    * Renames or moves the entry
+    *
+    * This is just a convinience alias to {@link dn()}
+    * to make your code more meaningful.
+    *
+    * @param string $newdn The new DN
+    * @return true
+    */
+    function move($newdn)
+    {
+        return $this->dn($newdn);
     }
 
     /**
@@ -349,6 +369,7 @@ class Net_LDAP_Entry extends PEAR
     * Alias function of getValue for perl-ldap interface
     *
     * @see getValue()
+    * @return string|array|PEAR_Error
     */
     function get_value()
     {
@@ -471,33 +492,44 @@ class Net_LDAP_Entry extends PEAR
         // therefore this has to be a simple list of attribute names to delete
         if (is_numeric(key($attr))) {
             foreach ($attr as $name) {
-                $name = $this->_getAttrName($name);
-                if ($this->exists($name)) {
-                    $this->_changes["delete"][$name] = null;
-                    unset($this->_attributes[$name]);
+                if (is_array($name)) {
+                    // someone mixed modes (list mode but specific values given!)
+                    $del_attr_name = array_search($name, $attr);
+                    $this->delete(array($del_attr_name => $name));
+                } else {
+                    $name = $this->_getAttrName($name);
+                    if ($this->exists($name)) {
+                        $this->_changes["delete"][$name] = null;
+                        unset($this->_attributes[$name]);
+                    }
                 }
             }
         } else {
             // Here we have a hash with "attributename" => "value to delete"
             foreach ($attr as $name => $values) {
-                // get the correct attribute name
-                $name = $this->_getAttrName($name);
-                if ($this->exists($name)) {
-                    if (false == is_array($values)) {
-                        $values = array($values);
-                    }
-                    // save values to be deleted
-                    if (empty($this->_changes["delete"][$name])) {
-                        $this->_changes["delete"][$name] = array();
-                    }
-                    $this->_changes["delete"][$name] =
-                        array_merge($this->_changes["delete"][$name], $values);
-                    foreach ($values as $value) {
-                        // find the key for the value that should be deleted
-                        $key = array_search($value, $this->_attributes[$name]);
-                        if (false !== $key) {
-                            // delete the value
-                            unset($this->_attributes[$name][$key]);
+                if (is_int($name)) {
+                    // someone mixed modes and gave us just an attribute name
+                    $this->delete($values);
+                } else {
+                    // get the correct attribute name
+                    $name = $this->_getAttrName($name);
+                    if ($this->exists($name)) {
+                        if (false == is_array($values)) {
+                            $values = array($values);
+                        }
+                        // save values to be deleted
+                        if (empty($this->_changes["delete"][$name])) {
+                            $this->_changes["delete"][$name] = array();
+                        }
+                        $this->_changes["delete"][$name] =
+                            array_merge($this->_changes["delete"][$name], $values);
+                        foreach ($values as $value) {
+                            // find the key for the value that should be deleted
+                            $key = array_search($value, $this->_attributes[$name]);
+                            if (false !== $key) {
+                                // delete the value
+                                unset($this->_attributes[$name][$key]);
+                            }
                         }
                     }
                 }
@@ -624,7 +656,7 @@ class Net_LDAP_Entry extends PEAR
             }
             $parent = Net_LDAP_Util::canonical_dn($parent);
 
-            // rename
+            // rename/move
             if (false == @ldap_rename($link, $this->_dn, $child, $parent, true)) {
                 return PEAR::raiseError("Entry not renamed: " .
                                         @ldap_error($link), @ldap_errno($link));
@@ -711,7 +743,8 @@ class Net_LDAP_Entry extends PEAR
     function &getLDAP()
     {
         if (!is_a($this->_ldap, 'Net_LDAP')) {
-            return PEAR::raiseError("LDAP is not a valid Net_LDAP object");
+            $err = new PEAR_Error('LDAP is not a valid Net_LDAP object');
+            return $err;
         } else {
             return $this->_ldap;
         }
@@ -738,6 +771,21 @@ class Net_LDAP_Entry extends PEAR
         }
     }
 
+    /**
+    * Marks the entry as new.
+    *
+    * If an Entry is marked as new, it will be added to the directory when
+    * calling {@link update()}. This method is mainly intendet for internal
+    * Net_LDAP package usage, so if you use it, use it with care.
+    *
+    * @access private
+    * @param boolean $mark Value to set, defaults to "true"
+    */
+    function _markAsNew($mark = true)
+    {
+        $this->_new = ($mark)? true : false;
+    }
+
     /**
     * Applies a regular expression onto a single- or multivalued attribute (like preg_match())
     *
@@ -802,5 +850,49 @@ class Net_LDAP_Entry extends PEAR
         }
         return $match;
     }
+
+    /**
+    * Is this entry going to be deleted once update() is called?
+    *
+    * @return boolean
+    */
+    function willBeDeleted()
+    {
+        return $this->_delete;
+    }
+
+    /**
+    * Is this entry going to be moved once update() is called?
+    *
+    * @return boolean
+    */
+    function willBeMoved()
+    {
+        return ($this->dn() !== $this->currentDN());
+    }
+
+    /**
+    * Returns always the original DN
+    *
+    * If an entry will be moved but {@link update()} was not called,
+    * {@link dn()} will return the new DN. This method however, returns
+    * always the current active DN.
+    *
+    * @return string
+    */
+    function currentDN()
+    {
+        return $this->_dn;
+    }
+
+    /**
+    * Returns the attribute changes to be carried out once update() is called
+    *
+    * @return array
+    */
+    function getChanges()
+    {
+        return $this->_changes;
+    }
 }
 ?>
\ No newline at end of file
