@@ -38,7 +38,7 @@ require_once 'Util.php';
 * @package  Net_LDAP
 * @author   Benedikt Hallinger <beni@php.net>
 * @license  http://www.gnu.org/copyleft/lesser.html LGPL
-* @version  CVS: $Id: Filter.php,v 1.24 2007/10/25 10:59:04 beni Exp $
+* @version  CVS: $Id: Filter.php,v 1.26 2007/12/17 06:49:27 beni Exp $
 * @link     http://pear.php.net/package/Net_LDAP/
 */
 class Net_LDAP_Filter extends PEAR
@@ -382,9 +382,14 @@ class Net_LDAP_Filter extends PEAR
                 print($filter_str);
             }
         } else {
-            $res = fwrite($FH, $this->asString());
-            if ($res == false) {
-                return PEAR::raiseError("Unable to write filter string to filehandle \$FH!");
+            $filter_str = $this->asString();
+            if (PEAR::isError($filter_str)) {
+                return $filter_str;
+            } else {
+                $res = @fwrite($FH, $this->asString());
+                if ($res == false) {
+                    return PEAR::raiseError("Unable to write filter string to filehandle \$FH!");
+                }
             }
         }
         return true;
@@ -403,13 +408,13 @@ class Net_LDAP_Filter extends PEAR
     * For upward compatibiliy reasons you are strongly encouraged to use the escape
     * methods provided by the Net_LDAP_Util class.
     *
-    * @param string $string Any string who should be escaped
+    * @param string $value Any string who should be escaped
     *
     * @static
     * @return string         The string $string, but escaped
     * @deprecated  Do not use this method anymore, instead use Net_LDAP_Util::escape_filter_value() directly
     */
-    function escape($string)
+    function escape($value)
     {
         $return = Net_LDAP_Util::escape_filter_value(array($value));
         return $return[0];
