@@ -11,7 +11,7 @@ require_once 'PEAR.php';
 * @author   Tarjej Huse <tarjei@bergfald.no>
 * @author   Benedikt Hallinger <beni@php.net>
 * @license  http://www.gnu.org/copyleft/lesser.html LGPL
-* @version  CVS $Id: Search.php,v 1.26 2007/10/24 06:29:32 beni Exp $
+* @version  CVS $Id: Search.php,v 1.31 2008/03/19 08:10:54 beni Exp $
 * @link     http://pear.php.net/package/Net_LDAP/
 */
 class Net_LDAP_Search extends PEAR
@@ -119,7 +119,7 @@ class Net_LDAP_Search extends PEAR
     }
 
     /**
-    * Returns an assosiative array of entry objects
+    * Returns an array of entry objects
     *
     * @return array Array of entry objects.
     */
@@ -472,6 +472,16 @@ class Net_LDAP_Search extends PEAR
     {
         return $this->_searchedAttrs;
     }
+
+    /**
+    * Tells if this search exceeds a sizelimit
+    *
+    * @return boolean
+    */
+    function sizeLimitExceeded()
+    {
+        return ($this->getErrorCode() == 4);
+    }
 }
 
 ?>
\ No newline at end of file
